<?php
/**
 * Payment Modul fur s-publicservices.de GiroConnect
 *
 * PHP version 5
 *
 * @category  Payment
 * @package   Giroconnect
 * @author    OCS
 * @copyright 2022 s-publicservices.de
 * @license   http://opensource.org/licenses/gpl-2.0.php GNU Public License *
 * @version   5.0.1
 * @link      http://s-publicservices.de
 * @link      http://www.gambio.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
// Konfiguration laden.
chdir('../../');
include_once 'includes/application_top.php';

if (file_exists(DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php') === true) {
  include_once DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
}
else {
  include_once 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
}

GiroCheckout_SDK_Config::getInstance()->setConfig('DEBUG_LOG_PATH', DIR_FS_CATALOG . 'logfiles/girocheckout');
define('COMPARE_SHOP_VERSION', 'v4.1');
include(DIR_FS_CATALOG . 'release_info.php');

$lang = 'de';
if (isset($_SESSION['language_code'])) {
  if (substr($_SESSION['language_code'], 0, 2) == 'en') {
    $lang = 'en';
  }
}
$paymentSuccess = false;
$paymentMsg = '';

// Parameter aus URL lesen.
$data = $_GET;
$pm = $data['pm'];

// Verbindung zur Datenbank aufbauen.
if (!xtc_db_connect()) {
  // Benachrichtigung soll nochmals erfolgen.
  header('HTTP/1.1 503 Service Unavailable');
  exit;
}

// Konfigurations-Parameter laden.
if (version_compare($gx_version, COMPARE_SHOP_VERSION, '>=')) {
  $configuration_query = xtc_db_query('select `key` as cfgKey, `value` as cfgValue from `gx_configurations`');
}
else {
  $configuration_query = xtc_db_query('select configuration_key as cfgKey,configuration_value as cfgValue from ' . TABLE_CONFIGURATION);
}

while ($configuration = xtc_db_fetch_array($configuration_query)) {
  define($configuration['cfgKey'], $configuration['cfgValue']);
}

if (defined('MODULE_PAYMENT_' . strtoupper($pm) . '_MERCHANTID')) {
  $merchantId = (int)constant('MODULE_PAYMENT_' . strtoupper($pm) . '_MERCHANTID');
}
if (defined('MODULE_PAYMENT_' . strtoupper($pm) . '_PROJECTID')) {
  $projectId = (int)constant('MODULE_PAYMENT_' . strtoupper($pm) . '_PROJECTID');
}
if (defined('MODULE_PAYMENT_' . strtoupper($pm) . '_PASSWORD')) {
  $projectPassword = constant('MODULE_PAYMENT_' . strtoupper($pm) . '_PASSWORD');
}
if (defined("MODULE_PAYMENT_" . strtoupper($pm) . "_DELETE_ORDER")) {
  $paymentDelOrder = strtolower(constant('MODULE_PAYMENT_' . strtoupper($pm) . '_DELETE_ORDER'));
  $delete_order = (($paymentDelOrder == 'true') ? true : false);
}

try {
  switch ($pm) {
    case "gc_giropay":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('giropayTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_creditcard":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('creditCardTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_ideal":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('idealPayment');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_eps":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('epsTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_paydirekt":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('paydirektTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_sofortuw":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('sofortuwTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_directdebit":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('directDebitTransactionWithPaymentPage');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_maestro":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('maestroTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    case "gc_bluecode":
      //Get the notification
      $notify = new GiroCheckout_SDK_Notify('blueCodeTransaction');
      $notify->setSecret($projectPassword);
      $notify->parseNotification($data);
      break;

    default:
      $notify->sendBadRequestStatus(); //400
      $notify->setNotifyResponseParam('Result', 'ERROR');
      $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: Payment method not valid.');
      $notify->setNotifyResponseParam('MailSent', '');
      $notify->setNotifyResponseParam('OrderId', '');
      $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
      echo $notify->getNotifyResponseStringJson();
      exit;
      break;
  }

  //loads transaction
  if (isset($data['gcMerchantTxId'])) {
    $gc_tx_id = $data['gcMerchantTxId'];

    //Get order_id from gc_transactions TABLE
    $transaction_query = xtc_db_query("SELECT order_id, tx_status FROM gc_transactions WHERE tx_id  = " . xtc_db_input($gc_tx_id));
    $transaction = xtc_db_fetch_array($transaction_query);
    $transactionId = $transaction["order_id"];
    $transactionStatus = $transaction["tx_status"];
  }

  //does transaction exist
  if (!isset($transactionId)) { // Benachrichtigung soll nochmals erfolgen.
    $notify->sendServiceUnavailableStatus(); //503
    $notify->setNotifyResponseParam('Result', 'ERROR');
    $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: Transaction not found.');
    $notify->setNotifyResponseParam('MailSent', '');
    $notify->setNotifyResponseParam('OrderId', '');
    $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
    echo $notify->getNotifyResponseStringJson();
    exit;
  }
  else {
    $orderId = $transaction["order_id"];

    $order_status_query = xtc_db_query("SELECT IF(COUNT(*) > 0, 'open', 'notified') AS payment_status FROM gc_transactions WHERE order_id  = " . $orderId . " AND tx_status = 1");
    $order_status = xtc_db_fetch_array($order_status_query);
    $payment_status = $order_status["payment_status"];
  }

  //check if payment was successful.
  if (!$notify->paymentSuccessful()) {
    $iReturnCode = $notify->getResponseParam('gcResultPayment');
    $newStatus = constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_ERROR');
    $paymentMsg = utf8_decode($notify->getResponseMessage($iReturnCode, strtoupper($lang)));

    if ($pm === "gc_giropay") {
      if (!$notify->avsSuccessful() && $notify->getResponseParam('gcResultAVS') != "4900") {
        $iReturnCodeAVS = $notify->getResponseParam('gcResultAVS');
        $paymentAVSMsg = $notify->getResponseMessage($iReturnCodeAVS, strtoupper($lang));
        if (!empty($paymentAVSMsg)) {
          $paymentMsg .= " (" . $paymentAVSMsg . ")";
        }
      }
    }

    $strGcBackendTxId = $notify->getResponseParam('gcBackendTxId');
    $strGcReference = $notify->getResponseParam('gcReference');
    $comment_order_status_history = $pm . ' (txId' . xtc_db_input($gc_tx_id) . ') payment error:  ' . utf8_decode($paymentMsg);
    $sStatus = "3";
  }
  else {
    $iReturnCode = $notify->getResponseParam('gcResultPayment');
    $paymentSuccess = true;
    $newStatus = constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_OK');
    $paymentMsg = utf8_decode($notify->getResponseMessage($iReturnCode, strtoupper($lang)));
    $strGcBackendTxId = $notify->getResponseParam('gcBackendTxId');
    $strGcReference = $notify->getResponseParam('gcReference');
    $comment_order_status_history = 'payment with ' . $pm . ' (txId' . xtc_db_input($gc_tx_id) . ') successful';
    $sStatus = "2";
  }
}
catch (Exception $e) {
  $iReturnCode = 5100;
  $newStatus = constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_ERROR');
  $paymentMsg = utf8_decode(GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, strtoupper($lang)));
  $strGcBackendTxId = '';
  $strGcReference = '';
  $comment_order_status_history = $pm . ' (txId' . xtc_db_input($gc_tx_id) . ') payment error:  ' . utf8_decode($paymentMsg);
  $sStatus = "3";
}

if ($payment_status == 'notified') { // Stops script, if payment already marked as successful.
  $notify->sendOkStatus(); //200
  $notify->setNotifyResponseParam('Result', 'ERROR'); //dejar
  $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: order already marked as paid. (a successful transaction exists for this order).');
  $notify->setNotifyResponseParam('MailSent', '');
  $notify->setNotifyResponseParam('OrderId', $transactionId);
  $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
  echo $notify->getNotifyResponseStringJson();
  exit;
}

// Pr�fen, ob Bestellung existiert.
$order_query = xtc_db_query('select * from ' . TABLE_ORDERS . " where orders_id = " . xtc_db_input((int)$transactionId));

if (xtc_db_num_rows($order_query) < 1) {
  // Bestellung nicht gefunden, Benachrichtigung soll nicht mehr erfolgen.
  $notify->sendBadRequestStatus(); //400
  $notify->setNotifyResponseParam('Result', 'ERROR');
  $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: order not found.');
  $notify->setNotifyResponseParam('MailSent', '');
  $notify->setNotifyResponseParam('OrderId', $transactionId);
  $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
  echo $notify->getNotifyResponseStringJson();
  exit;
}

// Bestell-Status in Tabelle orders updaten.
if (!xtc_db_query("update " . TABLE_ORDERS . " set orders_status = " . xtc_db_input($newStatus) . ", last_modified = now() where orders_id = " . xtc_db_input((int)$transactionId) . " AND payment_class = '" . xtc_db_input($pm) . "'")) {
  $notify->sendBadRequestStatus(); //400
  $notify->setNotifyResponseParam('Result', 'ERROR');
  $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: order update error.');
  $notify->setNotifyResponseParam('MailSent', '');
  $notify->setNotifyResponseParam('OrderId', $transactionId);
  $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
  echo $notify->getNotifyResponseStringJson();
  exit;
}

xtc_db_query("update gc_transactions set time_finished = " . strtotime("now") . ", reference = '" . xtc_db_input($strGcReference) . "', backend_tx_id = '" . xtc_db_input($strGcBackendTxId) . "', result_payment = " . xtc_db_input($iReturnCode) . ", result_payment_msg = '" . xtc_db_input($paymentMsg) . "', tx_status = " . xtc_db_input($sStatus) . " where tx_id = '" . xtc_db_input($gc_tx_id) . "'");

if ($transactionStatus == "1") {
// Eintrag zur Historie der Bestellung hinzuf�gen.
  $sql_data_array = array(
    'orders_id' => (int)$transactionId,
    'orders_status_id' => $newStatus,
    'date_added' => 'now()',
    'customer_notified' => (int)$paymentSuccess,
    'comments' => $comment_order_status_history
  );

  if (!xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array)) {
    $notify->sendBadRequestStatus(); //400
    $notify->setNotifyResponseParam('Result', 'ERROR');
    $notify->setNotifyResponseParam('ErrorMessage', 'ERROR: order history update error.');
    $notify->setNotifyResponseParam('MailSent', '');
    $notify->setNotifyResponseParam('OrderId', $transactionId);
    $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
    echo $notify->getNotifyResponseStringJson();
    exit;
  }
}

if ($paymentSuccess) {

  //Check if new class SendOrderProcess (added with version 2.1.x.x) exists to send a confirmation mail
  if (MainFactory::load_class('SendOrderProcess') === false) {
    //add the customer_id to the session in order to send the confirmation mail.

    $order_values = xtc_db_fetch_array($order_query);
    $_SESSION["customer_id"] = $order_values["customers_id"];

    // send email
    $smarty = new Smarty;
    require_once DIR_WS_CLASSES . 'order.php';
    $insert_id = $transactionId;
    include_once 'send_order.php';
  }
  else {
    //If exists, then use new class SendOrderProcess (added with version 2.1.x.x) to send a confirmation mail
    $oSendOrderProcess = MainFactory::create_object('SendOrderProcess');
    $oSendOrderProcess->set_('order_id', $transactionId);
    $oSendOrderProcess->proceed();
  }

  // return OK
  $notify->sendOkStatus();
  $notify->setNotifyResponseParam('Result', 'OK');
  $notify->setNotifyResponseParam('ErrorMessage', '');
  $notify->setNotifyResponseParam('MailSent', '');
  $notify->setNotifyResponseParam('OrderId', $transactionId);
  $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
  echo $notify->getNotifyResponseStringJson();
  exit;
  /*     * ************************************************************ */
}
else {
  if ($delete_order) {
    $check_query = xtc_db_query('select orders_id from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '" limit 1');
    if (xtc_db_num_rows($check_query) > 0) {
      xtc_db_query('delete from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
      xtc_db_query('delete from ' . TABLE_ORDERS_TOTAL . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
      xtc_db_query('delete from ' . TABLE_ORDERS_STATUS_HISTORY . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
      xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
      xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_ATTRIBUTES . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
      xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_DOWNLOAD . ' where orders_id = "' . xtc_db_input((int)$transactionId) . '"');
    }

    // return OK
    $notify->sendOkStatus();
    $notify->setNotifyResponseParam('Result', 'OK');
    $notify->setNotifyResponseParam('ErrorMessage', '');
    $notify->setNotifyResponseParam('MailSent', '');
    $notify->setNotifyResponseParam('OrderId', $transactionId);
    $notify->setNotifyResponseParam('CustomerId', $_SESSION['customer_id']);
    echo $notify->getNotifyResponseStringJson();
    exit;
    //200
  }
}

header('HTTP/1.1 200 OK');
exit;