<?php
/**
 * Payment Modul fur s-publicservices.de.
 *
 * @version   5.0.1
 * @author    OCS
 * @copyright 2011-2022 S-Public Services GmbH
 * @link      http://s-publicservices.de
 * @link      http://www.gambio.de
 * @license   http://opensource.org/licenses/gpl-2.0.php GNU Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
// Lib laden.
if (file_exists(DIR_FS_CATALOG . 'includes/classes/giroconnect/class.giroconnect.php') === true) {
  include_once DIR_FS_CATALOG . 'includes/classes/giroconnect/class.giroconnect.php';
}
else {
  include_once '../includes/classes/giroconnect/class.giroconnect.php';
}

if (file_exists(DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php') === true) {
  include_once DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
}
else {
  include_once '../includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
}

/**
 * Class to manage an ideal payment method.
 *
 * Extends the giroconnect_main
 *
 * @author S-Public Services GmbH
 * @package GiroCheckout
 * @copyright Copyright (c) 2022, S-Public Services GmbH
 */
class gc_ideal_ORIGIN extends giroconnect_main
{

  /**
   * Constructor method.
   * Set the payment type and payment method credentials.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return void
   */
  public function __construct() {
    global $order;

    $this->_setCode(self::PAYMENT_NAME_IDEAL);
    $this->_setLang();

    $this->title = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_TITLE');
    $this->description = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_DESCRIPTION');
    $this->description_checkout = xtc_image('images/manufacturers/girocheckout/gc_ideal.jpg', '', '', '', 'style="display: block;"') . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_DESCRIPTION');

    if (defined('MODULE_PAYMENT_' . strtoupper($this->code) . '_STATUS')) {
      $this->sort_order = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_SORT_ORDER');
      $paymentStatus = strtolower(giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_STATUS'));
      $this->enabled = (($paymentStatus == 'true') ? true : false);
      $this->redirect = true;
      $this->tmpOrders = true;
      $this->tmpStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ORDER_STATUS_ID_TEMP');
    }
    $this->Purpose = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_PURPOSE');

    if (defined('_VALID_XTC') === true) {
      if ($this->allowed != '') {
        $this->title .= ' (' . $this->allowed . ')';
      }
    }

    if (is_object($order)) {
      $this->update_status();
    }
  }
  //end gc_giropay_ORIGIN()

  /**
   * Set array with the title, description and fields to display.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return array
   */
  function selection() {
    $pic = xtc_image('images/manufacturers/girocheckout/gc_ideal.jpg', '', '', '', 'style="display: block;"');
    $desc = $this->description;

    return array(
      'id' => $this->code,
      'module' => $this->title,
      'description' => $desc
    );
  }
  //end selection()

  /**
   * Valid the payment datas inserted by the user.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return boolean
   */
  function pre_confirmation_check() {
    global $_POST;

    return true;
  }
  //end pre_confirmation_check()

  /**
   * Functions to do when the button is pressed.
   * Set hidden fields with payment datas.
   * Get the total.
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return void
   */
  function process_button() {
    global $order, $_POST, $xtPrice;

    if ($_SESSION['customers_status']['customers_status_show_price_tax'] == 0 && $_SESSION['customers_status']['customers_status_add_tax_ot'] == 1) {
      $total = $order->info['total'] + $order->info['tax'];
    }
    else {
      $total = $order->info['total'];
    }

    //sores total because of bug due to wrong pricing after an aborted transaction
    $_SESSION['giroconnect']['total'] = round($xtPrice->xtcCalculateCurrEx($total, $order->info['currency']), $xtPrice->get_decimal_places($order->info['currency']));


    $process_button_string = '<script>window.onload = function () {'
      . 'var gcclassname = document.getElementsByClassName("checkout-confirmation-submit");'
      . 'for (var i = 0; i < gcclassname.length; i++) {'
      . '    gcclassname[i].addEventListener("click", function(){'
      . 'this.disabled=true;'
      . 'var form = this.form;'
      . 'form.submit();'
      . '})'
      . '}}'
      . '</script>';

    return $process_button_string;
  }
  //end process_button()

  /**
   * Initialize the payment
   *
   * @author S-Public Services GmbH
   * @package GiroCheckout
   * @copyright Copyright (c) 2022, S-Public Services GmbH
   * @return boolean
   */
  function payment_action() {
    global $_POST, $xtPrice, $order, $insert_id;

    $order_id = $insert_id;

    if (empty($order_id)) {
      $order_id = $_SESSION['tmp_oID'];
      if (!isset($order)) {
        $order = new order($order_id);
      }
    }

    if (empty($order_id)) {
      $parts = explode('-', $_SESSION[$this->_gc_cartID]);
      $order_id = (int)end($parts);
    }

    // Fix for XTC Bug
    //
    // $order->info['total'] is in 'after_process' String with currency ->
    // 1.02 EUR
    // so it has to be set here
    // set total price

    if (class_exists('CheckoutProcessProcess')) {
      if (isset($order->info['pp_total'])) {
        $total = $order->info['pp_total'];
      }
      else {
        $total = $order->info['total'];
      }

      if ($_SESSION['customers_status']['customers_status_show_price_tax'] == 0 && $_SESSION['customers_status']['customers_status_add_tax_ot'] == 1) {
        $total = $total + $order->info['tax'];
      }

      $amount = round($total, $xtPrice->get_decimal_places($_SESSION['currency']));
    }
    else {
      $amount = $_SESSION['giroconnect']['total'];
    }

    $currency = $_SESSION['currency'];
    $transaction_id = $order_id;
    $purpose = $this->getPurpose($this->Purpose, $order, $transaction_id);
    $merchantId = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_MERCHANTID');
    $projectId = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_PROJECTID');
    $password = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_PASSWORD');
    $urlRedirect = str_replace('&amp;', '&', xtc_href_link('callback/giroconnect/gc_redirect.php', '', 'SSL')) . '?pm=' . $this->code;
    $urlNotify = str_replace('&amp;', '&', xtc_href_link('callback/giroconnect/gc_notify.php', '', 'SSL')) . '?pm=' . $this->code;
    $sourceId = $this->getGcSource();
    $initSuccess = true;
    $paymentDelOrder = strtolower(giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_DELETE_ORDER'));
    $delete_order = (($paymentDelOrder == 'true') ? true : false);

    //Insert new giroconnect transaction with init status.
    //xtc_db_query('delete from gc_transactions where order_id = "' . xtc_db_input((int) $transaction_id) . '"');
    $sql_data_array = array(
      'order_id' => $transaction_id,
      'rc' => 'null',
      'rc_msg' => 'null',
      'init_start' => strtotime("now"),
      'init_end' => 'null',
      'time_finished' => 'null',
      'reference' => 'null',
      'backend_tx_id' => 'null',
      'result_payment' => 'null',
      'tx_status' => 0,
      'payment_code' => $this->code,
      'comment' => $purpose,
    );

    xtc_db_perform('gc_transactions', $sql_data_array);
    $gc_tx_id = xtc_db_insert_id();

    try {

      //Sends request to Girocheckout.
      $request = new GiroCheckout_SDK_Request('idealPayment');
      $request->setSecret($password);
      $request->addParam('merchantId', $merchantId)
        ->addParam('projectId', $projectId)
        ->addParam('merchantTxId', $gc_tx_id)
        ->addParam('amount', $amount * 100)
        ->addParam('currency', $currency)
        ->addParam('purpose', $purpose)
        ->addParam('urlRedirect', $urlRedirect)
        ->addParam('urlNotify', $urlNotify)
        ->addParam('sourceId', $sourceId)
        ->addParam('orderId', $transaction_id)
        ->addParam('customerId', $_SESSION['customer_id'])
        ->submit();

      if (!$request->requestHasSucceeded()) {
        $iReturnCode = $request->getResponseParam('rc');
        $strResponseMsg = utf8_encode($request->getResponseMessage($iReturnCode, strtoupper($this->language)));
        $initSuccess = false;
        $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ORDER_STATUS_ID_ERROR');
      }
      else {
        $iReturnCode = $request->getResponseParam('rc');
        $strResponseMsg = utf8_encode($request->getResponseMessage($iReturnCode, strtoupper($this->language)));
        $strReference = $request->getResponseParam('reference');
        $strUrlRedirect = $request->getResponseParam('redirect');
        $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ORDER_STATUS_ID_TEMP');

        //Change giroconnect transaction to pending status.
        xtc_db_query("update gc_transactions set init_end = " . strtotime("now") . ", rc = " . xtc_db_input($iReturnCode) . ", reference = '" . xtc_db_input($strReference) . "', rc = " . xtc_db_input($iReturnCode) . ", rc_msg = '" . xtc_db_input($strResponseMsg) . "', tx_status = 1 where tx_id = '" . xtc_db_input($gc_tx_id) . "'");
      }
    }
    catch (Exception $e) {
      $strResponseMsg = utf8_encode(GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, strtoupper($this->language)));
      $iReturnCode = 5100;
      $initSuccess = false;
      $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ORDER_STATUS_ID_ERROR');
    }

    if (!$initSuccess) {

      //Change giroconnect transaction to failed status.
      xtc_db_query("update gc_transactions set init_end = " . strtotime("now") . ", rc = " . xtc_db_input($iReturnCode) . ", rc_msg = '" . xtc_db_input($strResponseMsg) . "', tx_status = 3 where tx_id = '" . xtc_db_input($gc_tx_id) . "'");

      // Eintrag zur Historie der Bestellung hinzuf�gen.
      $sql_data_array = array(
        'orders_id' => (int)$transaction_id,
        'orders_status_id' => $newStatus,
        'date_added' => 'now()',
        'customer_notified' => 0,
        'comments' => 'init ' . $this->code . ' (txId ' . $gc_tx_id . ') payment error: ' . utf8_decode($strResponseMsg)
      );
      xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

      // Bestell-Status in Tabelle orders updaten.
      xtc_db_query("update " . TABLE_ORDERS . " set orders_status = '" . xtc_db_input($newStatus) . "', last_modified = now() where orders_id = '" . xtc_db_input((int)$transaction_id) . "' AND payment_class = '" . xtc_db_input($this->code) . "'");

      if ($delete_order) {
        $check_query = xtc_db_query('select orders_id from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '" limit 1');
        if (xtc_db_num_rows($check_query) > 0) {
          xtc_db_query('delete from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
          xtc_db_query('delete from ' . TABLE_ORDERS_TOTAL . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
          xtc_db_query('delete from ' . TABLE_ORDERS_STATUS_HISTORY . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
          xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
          xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_ATTRIBUTES . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
          xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_DOWNLOAD . ' where orders_id = "' . xtc_db_input((int)$transaction_id) . '"');
        }
        unset($_SESSION['tmp_oID']);
      }

      xtc_redirect(str_replace('&amp;', '&', xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . urlencode($strResponseMsg), 'SSL', true, false)));

      return;
    }
    else {
      // Eintrag zur Historie der Bestellung hinzuf�gen.
      $sql_data_array = array(
        'orders_id' => (int)$transaction_id,
        'orders_status_id' => $newStatus,
        'date_added' => 'now()',
        'customer_notified' => 0,
        'comments' => 'payment with ' . $this->code . ' (txId ' . $gc_tx_id . ') initialized'
      );
      xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

      xtc_redirect($strUrlRedirect);
    }

    return true;
  }
  //end payment_action()
}

MainFactory::load_origin_class("gc_ideal");